@@ -111,15 +111,6 @@ xwl_unrealize_cursor(DeviceIntPtr device, ScreenPtr screen, CursorPtr cursor)
     return xwl_shm_destroy_pixmap(pixmap);
 }
 
-static void
-clear_cursor_frame_callback(struct xwl_cursor *xwl_cursor)
-{
-   if (xwl_cursor->frame_cb) {
-       wl_callback_destroy (xwl_cursor->frame_cb);
-       xwl_cursor->frame_cb = NULL;
-   }
-}
-
 static void
 frame_callback(void *data,
                struct wl_callback *callback,
@@ -127,7 +118,7 @@ frame_callback(void *data,
 {
     struct xwl_cursor *xwl_cursor = data;
 
-    clear_cursor_frame_callback(xwl_cursor);
+    xwl_cursor_clear_frame_cb(xwl_cursor);
     if (xwl_cursor->needs_update) {
         xwl_cursor->needs_update = FALSE;
         xwl_cursor->update_proc(xwl_cursor);
@@ -187,6 +178,18 @@ xwl_cursor_attach_pixmap(struct xwl_seat *xwl_seat,
     wl_surface_commit(xwl_cursor->surface);
 }
 
+Bool
+xwl_cursor_clear_frame_cb(struct xwl_cursor *xwl_cursor)
+{
+    if (xwl_cursor->frame_cb) {
+        wl_callback_destroy(xwl_cursor->frame_cb);
+        xwl_cursor->frame_cb = NULL;
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
 void
 xwl_seat_set_cursor(struct xwl_seat *xwl_seat)
 {
@@ -200,7 +203,7 @@ xwl_seat_set_cursor(struct xwl_seat *xwl_seat)
     if (!xwl_seat->x_cursor) {
         wl_pointer_set_cursor(xwl_seat->wl_pointer,
                               xwl_seat->pointer_enter_serial, NULL, 0, 0);
-        clear_cursor_frame_callback(xwl_cursor);
+        xwl_cursor_clear_frame_cb(xwl_cursor);
         xwl_cursor->needs_update = FALSE;
         return;
     }
@@ -238,7 +241,7 @@ xwl_tablet_tool_set_cursor(struct xwl_tablet_tool *xwl_tablet_tool)
         zwp_tablet_tool_v2_set_cursor(xwl_tablet_tool->tool,
                                       xwl_tablet_tool->proximity_in_serial,
                                       NULL, 0, 0);
-        clear_cursor_frame_callback(xwl_cursor);
+        xwl_cursor_clear_frame_cb(xwl_cursor);
         xwl_cursor->needs_update = FALSE;
         return;
     }
@@ -264,8 +267,15 @@ xwl_tablet_tool_set_cursor(struct xwl_tablet_tool *xwl_tablet_tool)
     xwl_cursor_attach_pixmap(xwl_seat, xwl_cursor, pixmap);
 }
 
+void
+xwl_cursor_release(struct xwl_cursor *xwl_cursor)
+{
+    wl_surface_destroy(xwl_cursor->surface);
+    xwl_cursor_clear_frame_cb(xwl_cursor);
+}
+
 static void
-xwl_seat_update_cursor(struct xwl_seat *xwl_seat)
+xwl_seat_update_all_cursors(struct xwl_seat *xwl_seat)
 {
     struct xwl_tablet_tool *xwl_tablet_tool;
 
@@ -285,7 +295,7 @@ xwl_seat_update_cursor_visibility(struct xwl_seat *xwl_seat)
 {
     xwl_seat->x_cursor = xwl_seat->pending_x_cursor;
     xwl_seat_cursor_visibility_changed(xwl_seat);
-    xwl_seat_update_cursor(xwl_seat);
+    xwl_seat_update_all_cursors(xwl_seat);
 }
 
 static void
@@ -339,7 +349,7 @@ xwl_set_cursor(DeviceIntPtr device,
         /* Cursor remains shown or hidden, apply the change immediately */
         xwl_set_cursor_free_timer(xwl_seat);
         xwl_seat->x_cursor = cursor;
-        xwl_seat_update_cursor(xwl_seat);
+        xwl_seat_update_all_cursors(xwl_seat);
         return;
     }
 
